//=============================================================================
// Drill_WindowSkillElement.js
//=============================================================================

/*:
 * @plugindesc [v1.1]        主菜单 - 技能窗口块元素
 * @author Drill_up
 *
 * @param ----窗口----
 * @default 
 * 
 * @param 是否应用于菜单技能窗口
 * @parent ----窗口----
 * @type boolean
 * @on 应用
 * @off 关闭
 * @desc true - 应用，false - 关闭。
 * @default true
 * 
 * @param 是否应用于战斗技能窗口
 * @parent ----窗口----
 * @type boolean
 * @on 应用
 * @off 关闭
 * @desc true - 应用，false - 关闭。
 * @default true
 * 
 * @param 是否开启闪烁指针
 * @parent ----窗口----
 * @type boolean
 * @on 开启
 * @off 关闭
 * @desc 窗口选中项的闪烁的白色方块的开关。(mog的菜单选项外框也会被隐藏。)
 * @default true
 *
 * @param ----元素组----
 * @default 
 * 
 * @param 选中元素偏移 X
 * @parent ----元素组----
 * @desc 选中的元素会自动移动到偏移的位置。x轴方向平移，单位像素。正数向右，负数向左。
 * @default 0
 * 
 * @param 选中元素偏移 Y
 * @parent ----元素组----
 * @desc 选中的元素会自动移动到偏移的位置。y轴方向平移，单位像素。正数向下，负数向上。
 * @default -20
 * 
 * @param 选中元素缩放 X
 * @parent ----元素组----
 * @desc 选中的元素会自动放大到指定比例，x轴左右缩放。1.00为默认大小。
 * @default 1.00
 * 
 * @param 选中元素缩放 Y
 * @parent ----元素组----
 * @desc 选中的元素会自动放大到指定比例，y轴上下缩放。1.00为默认大小。
 * @default 1.00
 * 
 * @param 元素列数-战斗界面
 * @parent ----元素组----
 * @type number
 * @min 0
 * @desc 元素的列数，将会覆盖其它插件设置的技能窗口列数。详细配置方法去看看"关于技能窗口块元素.docx"。
 * @default 4
 * 
 * @param 元素列数-菜单界面
 * @parent ----元素组----
 * @type number
 * @min 0
 * @desc 元素的列数，将会覆盖其它插件设置的技能窗口列数。详细配置方法去看看"关于技能窗口块元素.docx"。
 * @default 2
 * 
 * @param 元素高度
 * @parent ----元素组----
 * @type number
 * @min 0
 * @desc 元素占的高度大小。详细配置方法去看看"关于技能窗口块元素.docx"。
 * @default 90
 * 
 * @param 元素横向间距
 * @parent ----元素组----
 * @type number
 * @min 0
 * @desc 横向两个选项之间的间距。详细配置方法去看看"关于技能窗口块元素.docx"。
 * @default 12
 * 
 * @param 是否严格控制元素的区域
 * @parent ----元素组----
 * @type boolean
 * @on 严格控制
 * @off 关闭
 * @desc true - 严格控制，false - 关闭。严格控制下，元素的块内容如果超出了元素的高宽，将会被遮挡、剪去。
 * @default true
 *
 * @param ----背景块----
 * @default 
 *
 * @param 资源-元素背景
 * @parent ----背景块----
 * @desc 元素背景的图片资源。
 * @default 技能窗口-技能卡槽背景
 * @require 1
 * @dir img/battlehud/
 * @type file
 *
 * @param ----图标块----
 * @default 
 * 
 * @param 平移-图标 X
 * @parent ----图标块----
 * @type number
 * @min 0
 * @desc x轴方向平移，单位像素。0为贴在最左边。
 * @default 134
 * 
 * @param 平移-图标 Y
 * @parent ----图标块----
 * @type number
 * @min 0
 * @desc y轴方向平移，单位像素。0为贴在最上面。
 * @default 18
 * 
 * @param 图标大小
 * @parent ----图标块----
 * @desc 图标的比例大小，1.00为原大小。
 * @default 1.00
 *
 * @param ----技能名块----
 * @default 
 * 
 * @param 平移-技能名 X
 * @parent ----技能名块----
 * @type number
 * @min 0
 * @desc x轴方向平移，单位像素。0为贴在最左边。
 * @default 42
 * 
 * @param 平移-技能名 Y
 * @parent ----技能名块----
 * @type number
 * @min 0
 * @desc y轴方向平移，单位像素。0为贴在最上面。
 * @default 3
 * 
 * @param 技能名字体大小
 * @parent ----技能名块----
 * @type number
 * @min 0
 * @desc 技能名的字体大小。将会覆盖其它插件设置的该窗口字体大小。
 * @default 20
 * 
 * @param 技能块宽度
 * @parent ----技能名块----
 * @type number
 * @min 10
 * @desc 技能块区域的宽度。技能名如果太长，会根据宽度压缩。也与对齐方式相关。
 * @default 88
 *
 * @param 技能名对齐方式
 * @parent ----技能名块----
 * @type select
 * @option 左对齐
 * @value left
 * @option 居中
 * @value center
 * @option 右对齐
 * @value right
 * @desc 选项文本的对齐方式，left - 左对齐，center- 居中，right - 右对齐。
 * @default left
 *
 * @param ----消耗块----
 * @default 
 * 
 * @param 平移-消耗块 X
 * @parent ----消耗块----
 * @type number
 * @min 0
 * @desc x轴方向平移，单位像素。0为贴在最左边。
 * @default 42
 * 
 * @param 平移-消耗块 Y
 * @parent ----消耗块----
 * @type number
 * @min 0
 * @desc y轴方向平移，单位像素。0为贴在最上面。
 * @default 31
 * 
 * @param 消耗值字体大小
 * @parent ----消耗块----
 * @type number
 * @min 0
 * @desc 消耗值的字体大小。将会覆盖其它插件设置的该窗口字体大小。
 * @default 20
 * 
 * @param 消耗块宽度
 * @parent ----消耗块----
 * @type number
 * @min 10
 * @desc 消耗块区域的宽度。消耗值如果太长，会根据宽度压缩。也与对齐方式相关。
 * @default 88
 *
 * @param 消耗值对齐方式
 * @parent ----消耗块----
 * @type select
 * @option 左对齐
 * @value left
 * @option 居中
 * @value center
 * @option 右对齐
 * @value right
 * @desc 消耗值的对齐方式，left - 左对齐，center- 居中，right - 右对齐。
 * @default left
 * 
 * @param 魔法消耗文本颜色
 * @parent ----消耗块----
 * @type number
 * @min 0
 * @desc 魔法消耗的文本颜色，你可以输入100、200设置高级颜色，但前提是你使用了高级文本颜色插件。
 * @default 23
 * 
 * @param 怒气消耗文本颜色
 * @parent ----消耗块----
 * @type number
 * @min 0
 * @desc 怒气消耗的文本颜色，你可以输入100、200设置高级颜色，但前提是你使用了高级文本颜色插件。
 * @default 27
 * 
 * @param 无消耗文本
 * @parent ----消耗块----
 * @desc 魔法和怒气的消耗值为零情况下，显示的文本。可以为空。
 * @default -
 * 
 * @param 无消耗文本颜色
 * @parent ----消耗块----
 * @type number
 * @min 0
 * @desc 消耗值为零的文本颜色，你可以输入100、200设置高级颜色，但前提是你使用了高级文本颜色插件。
 * @default 23
 *
 * @help  
 * =============================================================================
 * +++ Drill_WindowSkillElement +++
 * 作者：Drill_up
 * 如果你有兴趣，也可以来看看我的mog中文全翻译插件哦ヽ(*。>Д<)o゜
 * https://rpg.blue/thread-409713-1-1.html
 * =============================================================================
 * 使得你可以针对技能窗口 内部元素 进行全自定义美化。
 * 详细配置方法去看看"关于技能窗口块元素.docx"。
 * ★★必须放在 全自定义技能界面、战斗角色窗口 插件的后面★★
 * 
 * -----------------------------------------------------------------------------
 * ----插件扩展
 * 该插件能单独使用，它只针对技能窗口 内部元素 进行美化。
 * 被扩展：
 *   - Drill_X_ElementSkillImage.js 主菜单-技能块元素的背景图片[扩展]
 *     通过该插件可以使得块元素中不同的技能，拥有不同的背景。
 * 
 * -----------------------------------------------------------------------------
 * ----关联文件
 * 先确保项目img文件夹下是否有battlehud文件夹！（img/battlehud）
 * 如果没有，需要自己建立。需要配置资源文件：
 *
 * 资源-元素背景
 *
 *
 * -----------------------------------------------------------------------------
 * ----更新日志
 * [v1.0]
 * 完成插件ヽ(*。>Д<)o゜
 * [v1.1]
 * 修复了在mog的战斗窗口中，宽度修改不了的bug。
 */
 
//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
//		临时全局变量	DrillUp.xxx
//		临时局部变量	无
//		存储数据变量	无
//		全局存储变量	无
//		覆盖重写方法	Window_BattleSkill.prototype.drawAllItems
//						Window_BattleSkill.prototype.drawItem
//
//插件记录：
//		静态界面，直接在Window_BattleSkill窗口的contents中画（contents就是个bitmap）。
//		静态的关系直接参照window的rmmv原相关方法画。
//
//		这里要推倒大部分，画sprite体现。
//		原理：
//			rmmv本体绘制，是在一个bitmap上面，重复画一堆内容，超过窗口的选项不画。（原本以为存在下拉图，结果完全不是）
//			rmmv本体每次上移下移动作，都会通过刷新drawItems，重新绘制当前的技能内容。
//			这里将bitmap换成sprite阵列。（面板最多放8个技能，那么就加8个sprite）
//				
//		绘制渐变颜色时，bitmap的刷子不知道为什么产生了偏移，置零了。目前解决方法是用 Bitmap.drill_elements_drawText 做识别。
//
//		Window_SkillList 是 Window_BattleSkill 的父类
//			采用覆写分配的方式来控制，（主要需要控制 父类被覆写，而确保子类 不被覆写）
//			（覆写弄的乱七八糟的……关注drill_xxx方法就好了。）
//
 
//=============================================================================
// ** 变量获取
//=============================================================================
　　var Imported = Imported || {};
　　Imported.Drill_WindowSkillElement = true;
　　var DrillUp = DrillUp || {}; 
    DrillUp.parameters = PluginManager.parameters('Drill_WindowSkillElement');

	DrillUp.bSkillWin_use_in_menu = String(DrillUp.parameters['是否应用于菜单技能窗口'] || "true") === "true";	
	DrillUp.bSkillWin_use_in_battle = String(DrillUp.parameters['是否应用于战斗技能窗口'] || "true") === "true";	
	DrillUp.bSkillWin_use_cursorRect = String(DrillUp.parameters['是否开启闪烁指针'] || "true") === "true";	
    DrillUp.bSkillWin_selected_pos_x = Number(DrillUp.parameters['选中元素偏移 X'] || 0);
    DrillUp.bSkillWin_selected_pos_y = Number(DrillUp.parameters['选中元素偏移 Y'] || -20);
    DrillUp.bSkillWin_selected_size_x = Number(DrillUp.parameters['选中元素缩放 X'] || 1.00);
    DrillUp.bSkillWin_selected_size_y = Number(DrillUp.parameters['选中元素缩放 Y'] || 1.00);
    DrillUp.bSkillWin_battle_col = Number(DrillUp.parameters['元素列数-战斗界面'] || 4);
    DrillUp.bSkillWin_menu_col = Number(DrillUp.parameters['元素列数-菜单界面'] || 2);
    DrillUp.bSkillWin_height = Number(DrillUp.parameters['元素高度'] || 90);
    DrillUp.bSkillWin_spacing = Number(DrillUp.parameters['元素横向间距'] || 12);
	DrillUp.bSkillWin_is_strict = String(DrillUp.parameters['是否严格控制元素的区域'] || "true") === "true";	
	
    DrillUp.bSkillWin_src_background = String(DrillUp.parameters['资源-元素背景'] || "");
    DrillUp.bSkillWin_icon_x = Number(DrillUp.parameters['平移-图标 X'] || 134);
    DrillUp.bSkillWin_icon_y = Number(DrillUp.parameters['平移-图标 Y'] || 18);
    DrillUp.bSkillWin_icon_size = Number(DrillUp.parameters['图标大小'] || 1.00);
    DrillUp.bSkillWin_skill_x = Number(DrillUp.parameters['平移-技能名 X'] || 42);
    DrillUp.bSkillWin_skill_y = Number(DrillUp.parameters['平移-技能名 Y'] || 3);
    DrillUp.bSkillWin_skill_fontsize = Number(DrillUp.parameters['技能名字体大小'] || 20);
    DrillUp.bSkillWin_skill_width = Number(DrillUp.parameters['技能块宽度'] || 88);
    DrillUp.bSkillWin_skill_align = String(DrillUp.parameters['技能名对齐方式'] || 'left');
	
    DrillUp.bSkillWin_cost_x = Number(DrillUp.parameters['平移-消耗块 X'] || 42);
    DrillUp.bSkillWin_cost_y = Number(DrillUp.parameters['平移-消耗块 Y'] || 31);
    DrillUp.bSkillWin_cost_fontsize = Number(DrillUp.parameters['消耗值字体大小'] || 20);
    DrillUp.bSkillWin_cost_width = Number(DrillUp.parameters['消耗块宽度'] || 88);
    DrillUp.bSkillWin_cost_align = String(DrillUp.parameters['消耗值对齐方式'] || 'left');
    DrillUp.bSkillWin_cost_mp_color = Number(DrillUp.parameters['魔法消耗文本颜色'] || 23);
    DrillUp.bSkillWin_cost_tp_color = Number(DrillUp.parameters['怒气消耗文本颜色'] || 27);
    DrillUp.bSkillWin_cost_no_cost = String(DrillUp.parameters['无消耗文本'] || "-");
    DrillUp.bSkillWin_cost_no_cost_color = Number(DrillUp.parameters['无消耗文本颜色'] || 23);
	
	
//=============================================================================
// ** ImageManager
//=============================================================================
ImageManager.loadBHud = function(filename) {
    return this.loadBitmap('img/battlehud/', filename, 0, true);
};		

//=============================================================================
// ** 窗口初始化（追加分配）
//=============================================================================
var _drill_bSkillWin_menu_initialize = Window_SkillList.prototype.initialize;
var _drill_bSkillWin_battle_initialize = Window_BattleSkill.prototype.initialize;

if( DrillUp.bSkillWin_use_in_menu ){
	Window_SkillList.prototype.initialize = function(x, y, width, height) {
		_drill_bSkillWin_menu_initialize.call(this, x, y, width, height);
		this.drill_skillSpriteInit();
	};
	Window_BattleSkill.prototype.initialize = function(x, y, width, height) {
		_drill_bSkillWin_battle_initialize.call(this, x, y, width, height);
	};
	if(Imported.MOG_SceneSkill){
		var _drill_bSkillWin_MOG_menu_initialize = Window_SkillListM.prototype.initialize;
		Window_SkillListM.prototype.initialize = function(x, y, width, height) {
			_drill_bSkillWin_MOG_menu_initialize.call(this, x, y, width, height);
			this.drill_skillSpriteInit();
		};
	}
}

if( DrillUp.bSkillWin_use_in_battle ){
	
	if( Imported.MOG_BattleHud ){	//mog角色窗口设置必须实时初始化
		Window_BattleSkill.prototype.initialize = function(x, y, width, height) {
			_drill_bSkillWin_battle_initialize.call(this, x, y, Moghunter.bhud_skill_width, Moghunter.bhud_skill_height);
			this.drill_skillSpriteInit();
		};
	}else{
		Window_BattleSkill.prototype.initialize = function(x, y, width, height) {
			_drill_bSkillWin_battle_initialize.call(this, x, y, width, height);
			this.drill_skillSpriteInit();
		};
	}
}

//==============================
// * 窗口sprite初始化
//==============================
Window_SkillList.prototype.drill_skillSpriteInit = function() {
	this._drill_sprites = [];
	this._drill_bitmaps = [];
	this._drill_bitmap_back = ImageManager.loadBHud(DrillUp.bSkillWin_src_background);
	
	//这里建立sprite层，根据rect的大小固定创建 sprite
	
	if( !this._optionField ){		//选项层
		this._optionField = new Sprite();
		this.addChild(this._optionField);
	}
	for( var j = 0; j < this.maxPageItems(); j++ ){	//画面就出现n*m矩阵，多余的部分不画。
		var rect = this.itemRect( j );
		var temp_sprite = new Sprite();
		temp_sprite.x = this.standardPadding() + rect.x;
		temp_sprite.y = this.standardPadding() + rect.y;
		temp_sprite._org_x = temp_sprite.x;
		temp_sprite._org_y = temp_sprite.y;
		
		if( DrillUp.bSkillWin_is_strict ){
			var temp_bitmap = new Bitmap( rect.width , rect.height );
		}else{
			var temp_bitmap = new Bitmap( this.width , this.height );
		}
		temp_bitmap.fontSize = this.standardFontSize();
		
		temp_sprite.bitmap = temp_bitmap;
		this._drill_bitmaps.push(temp_bitmap);
		this._drill_sprites.push(temp_sprite);
		this._optionField.addChild(temp_sprite);	
		
	}
}

//=============================================================================
// ** 窗口内容值初始化（覆写分配）
//=============================================================================
var _drill_bSkillWin_menu_itemHeight = Window_SkillList.prototype.itemHeight;
var _drill_bSkillWin_battle_itemHeight = Window_BattleSkill.prototype.itemHeight;
var _drill_bSkillWin_menu_maxCols = Window_SkillList.prototype.maxCols;
var _drill_bSkillWin_battle_maxCols = Window_BattleSkill.prototype.maxCols;
var _drill_bSkillWin_menu_spacing = Window_SkillList.prototype.spacing;
var _drill_bSkillWin_battle_spacing = Window_BattleSkill.prototype.spacing;

if( DrillUp.bSkillWin_use_in_menu ){
	Window_SkillList.prototype.itemHeight = function() {
		return DrillUp.bSkillWin_height;
	};
	Window_SkillList.prototype.maxCols = function() {
		return DrillUp.bSkillWin_menu_col;
	};
	Window_SkillList.prototype.spacing = function() {
		return DrillUp.bSkillWin_spacing;
	};
	Window_BattleSkill.prototype.itemHeight = function() {
		return _drill_bSkillWin_battle_itemHeight.call(this);
	};
	Window_BattleSkill.prototype.maxCols = function() {
		return _drill_bSkillWin_battle_maxCols.call(this);
	};
	Window_BattleSkill.prototype.spacing = function() {
		return _drill_bSkillWin_battle_spacing.call(this);
	};
	if(Imported.MOG_SceneSkill){
		Window_SkillListM.prototype.itemHeight = function() {
			return DrillUp.bSkillWin_height;
		};
		Window_SkillListM.prototype.maxCols = function() {
			return DrillUp.bSkillWin_menu_col;
		};
		Window_SkillListM.prototype.spacing = function() {
			return DrillUp.bSkillWin_spacing;
		};
	}
}

if( DrillUp.bSkillWin_use_in_battle ){
	Window_BattleSkill.prototype.itemHeight = function() {
		return DrillUp.bSkillWin_height;
	};
	Window_BattleSkill.prototype.maxCols = function() {
		return DrillUp.bSkillWin_battle_col;
	};
	Window_BattleSkill.prototype.spacing = function() {
		return DrillUp.bSkillWin_spacing;
	};
}

//==============================
// * bitmap窗口动作 刷新（覆写分配）
//==============================
var _drill_bSkillWin_menu_drawAllItems = Window_SkillList.prototype.drawAllItems;
var _drill_bSkillWin_battle_drawAllItems = Window_BattleSkill.prototype.drawAllItems;

if( DrillUp.bSkillWin_use_in_menu ){
	Window_SkillList.prototype.drawAllItems = function() {
		this.drill_drawAllItems();
	};
	Window_BattleSkill.prototype.drawAllItems = function() {
		_drill_bSkillWin_battle_drawAllItems.call( this );
	};
	if(Imported.MOG_SceneSkill){
		Window_SkillListM.prototype.drawAllItems = function() {
			this.drill_drawAllItems();
		};
	}
}

if( DrillUp.bSkillWin_use_in_battle ){
	Window_BattleSkill.prototype.drawAllItems = function() {
		this.drill_drawAllItems();
	};
}

//==============================
// * bitmap窗口动作 刷新
//==============================
Window_SkillList.prototype.drill_drawAllItems = function() {
	var topIndex = this.topIndex();
	
	for (var i = 0; i < this.maxPageItems(); i++) {
		var index = topIndex + i;
		var cur_bitmap = this._drill_bitmaps[i];
		if (index < this.maxItems()) {
			this.drawItem(index,cur_bitmap);
		}else{
			cur_bitmap.clear();
		}
	}
};

//==============================
// * bitmap绘制（覆写分配）
//==============================
var _drill_bSkillWin_menu_drawItem = Window_SkillList.prototype.drawItem;
var _drill_bSkillWin_battle_drawItem = Window_BattleSkill.prototype.drawItem;

if( DrillUp.bSkillWin_use_in_menu ){
	Window_SkillList.prototype.drawItem = function(index,cur_bitmap) {
		this.drill_drawItem(index,cur_bitmap);
	};
	Window_BattleSkill.prototype.drawItem = function(index,cur_bitmap) {
		_drill_bSkillWin_battle_drawItem.call( this,index,cur_bitmap );
	};
	if(Imported.MOG_SceneSkill){
		Window_SkillListM.prototype.drawItem = function(index,cur_bitmap) {
			this.drill_drawItem(index,cur_bitmap);
		};
	}
}

if( DrillUp.bSkillWin_use_in_battle ){
	Window_BattleSkill.prototype.drawItem = function(index,cur_bitmap) {
		this.drill_drawItem(index,cur_bitmap);
	};
}

//==============================
// * bitmap绘制
//==============================
Window_SkillList.prototype.drill_drawItem = function(index,cur_bitmap) {
    var skill = this._data[index];
    if (skill) {
        var rect = this.itemRect(index);
        rect.width -= this.textPadding();
		cur_bitmap._disabled = !this.isEnabled(skill);
		cur_bitmap.paintOpacity = this.isEnabled(skill) ? 255 : this.translucentOpacity();	//	不可用的技能透明度控制
		cur_bitmap.clear();
        this.drill_s_drawBlock(cur_bitmap, skill );
		cur_bitmap.paintOpacity = 255 ;
    }
};
Window_SkillList.prototype.drill_s_drawBlock = function(cur_bitmap, skill ) {
	
    if (skill) {
		cur_bitmap.drill_elements_drawText = null;	//绘制颜色特殊位置修正
        this.resetTextColor();
        
		this.drill_s_drawBackground(cur_bitmap, skill);		//绘制背景
		//cur_bitmap.fillRect(0, 0 , cur_bitmap.width, cur_bitmap.height, "#000");
		
		this.drill_s_drawIcon(cur_bitmap, skill);			//绘制图标
		
		
		var name_x = DrillUp.bSkillWin_skill_x;		//绘制技能名
		var name_y = DrillUp.bSkillWin_skill_y;
		var name_width = DrillUp.bSkillWin_skill_width; //cur_bitmap.width;
		var name_align = DrillUp.bSkillWin_skill_align;
		cur_bitmap.fontSize = DrillUp.bSkillWin_skill_fontsize;
		if(Imported.Drill_ItemTextColor){
			var temp_c = $gameSystem._drill_color_skills[ skill.id ];
			if( temp_c != "" ){ 
				cur_bitmap.textColor = temp_c ;
				cur_bitmap.drill_elements_drawText = true;
			}
		}
		cur_bitmap.drawText( String(skill.name), name_x, name_y, name_width, this.lineHeight(), name_align);
		cur_bitmap.textColor = "#ffffff" ;
		
		var cost_x = DrillUp.bSkillWin_cost_x;		// 绘制消耗值
		var cost_y = DrillUp.bSkillWin_cost_y;
		var cost_width = DrillUp.bSkillWin_cost_width; //cur_bitmap.width;
		var cost_align = DrillUp.bSkillWin_cost_align;
		cur_bitmap.fontSize = DrillUp.bSkillWin_cost_fontsize;
		if (this._actor.skillTpCost(skill) > 0) {		
			cur_bitmap.textColor = this.textColor(DrillUp.bSkillWin_cost_tp_color) ;
			cur_bitmap.drill_elements_drawText = true;
			cur_bitmap.drawText( String(this._actor.skillTpCost(skill)), cost_x, cost_y, cost_width,  this.lineHeight(), cost_align);
			
		} else if (this._actor.skillMpCost(skill) > 0) {
			cur_bitmap.textColor = this.textColor(DrillUp.bSkillWin_cost_mp_color) ;
			cur_bitmap.drill_elements_drawText = true;
			cur_bitmap.drawText( String(this._actor.skillMpCost(skill)), cost_x, cost_y, cost_width,  this.lineHeight(), cost_align);
		
		} else if ( this._actor.skillTpCost(skill) == 0 || this._actor.skillMpCost(skill) == 0) {
			cur_bitmap.textColor = this.textColor(DrillUp.bSkillWin_cost_no_cost_color) ;
			cur_bitmap.drill_elements_drawText = true;
			cur_bitmap.drawText( String(DrillUp.bSkillWin_cost_no_cost), cost_x, cost_y, cost_width,  this.lineHeight(), cost_align);
		}
		cur_bitmap.textColor = "#ffffff" ;
    }
};

Window_SkillList.prototype.drill_s_drawBackground = function(cur_bitmap, skill ) {
	
	//绘制背景（背景后期可能会扩展，这里从单独拿出来）
	var back_x = 0;
	var back_y = 0;
	var back_w = this._drill_bitmap_back.width;
	var back_h = this._drill_bitmap_back.height;
	cur_bitmap.blt( this._drill_bitmap_back,  back_x, back_y, back_w, back_h,  0,0, back_w, back_h);
}
Window_SkillList.prototype.drill_s_drawIcon = function(cur_bitmap, skill ) {
	
	//绘制图标（背景后期可能会扩展，这里从单独拿出来）
	var pbitmap = ImageManager.loadSystem('IconSet');
	var pw = Window_Base._iconWidth ;
	var ph = Window_Base._iconHeight ;
	var px = skill.iconIndex % 16 * pw ;
	var py = Math.floor(skill.iconIndex / 16) * ph;
	var icon_x = DrillUp.bSkillWin_icon_x;
	var icon_y = DrillUp.bSkillWin_icon_y;
	var icon_size = DrillUp.bSkillWin_icon_size;
	cur_bitmap._context.imageSmoothingEnabled = false;
	cur_bitmap.blt( pbitmap,  px, py, pw, ph,  icon_x,icon_y, pw*icon_size, ph*icon_size);
	cur_bitmap._context.imageSmoothingEnabled = true;
}

//==============================
// * sprite动作（追加分配）
//==============================
var _drill_bSkillWin_menu_update = Window_SkillList.prototype.update;
var _drill_bSkillWin_battle_update = Window_BattleSkill.prototype.update;

if( DrillUp.bSkillWin_use_in_menu ){
	Window_SkillList.prototype.update = function() {
		_drill_bSkillWin_menu_update.call(this);
		this.drill_skillSpriteUpdate();
	}
	Window_BattleSkill.prototype.update = function() {
		_drill_bSkillWin_battle_update.call(this);
	};
	if(Imported.MOG_SceneSkill){
		var _drill_bSkillWin_MOG_menu_update = Window_SkillListM.prototype.update;
		Window_SkillListM.prototype.update = function(x, y, width, height) {
			_drill_bSkillWin_MOG_menu_update.call(this, x, y, width, height);
			this.drill_skillSpriteUpdate();
		};
	}
}

if( DrillUp.bSkillWin_use_in_battle ){
	Window_BattleSkill.prototype.update = function() {
		_drill_bSkillWin_battle_update.call(this);
		this.drill_skillSpriteUpdate();
	}
}

//==============================
// * sprite动作
//==============================
Window_SkillList.prototype.drill_skillSpriteUpdate = function() {
	for(var i = 0;i< this._drill_sprites.length; i++){
		var temp_sprite = this._drill_sprites[i];
		if(  this._index == i + this.topIndex() 
			&& !(temp_sprite.bitmap._disabled) ){
			this.drill_button_move_to(temp_sprite,
				temp_sprite._org_x + DrillUp.bSkillWin_selected_pos_x,
				temp_sprite._org_y + DrillUp.bSkillWin_selected_pos_y,
				3);
			this.drill_scale_move_to(temp_sprite,DrillUp.bSkillWin_selected_size_x,'x',0.01);
			this.drill_scale_move_to(temp_sprite,DrillUp.bSkillWin_selected_size_y,'y',0.01);
		}else{
			this.drill_button_move_to(temp_sprite,temp_sprite._org_x,temp_sprite._org_y,3);
			this.drill_scale_move_to(temp_sprite,1.00,'x',0.01);
			this.drill_scale_move_to(temp_sprite,1.00,'y',0.01);
		}
		if(this.contentsOpacity != null){
			temp_sprite.opacity = this.contentsOpacity;
		}
	}
}

//==============================
// * 按钮变化工具方法
//==============================
Window_SkillList.prototype.drill_button_move_to = function(sprite,x,y,speed) {
	var dx = sprite.x - x;
	var dy = sprite.y - y;
	if( dx < 0 ){ sprite.x += speed; }
	if( dx > 0 ){ sprite.x -= speed; }
	if( dy < 0 ){ sprite.y += speed; }
	if( dy > 0 ){ sprite.y -= speed; }
		
	if( Math.abs(dx) <= speed ){ sprite.x = x; }
	if( Math.abs(dy) <= speed ){ sprite.y = y; }
}

Window_SkillList.prototype.drill_scale_move_to = function(sprite,s,type,speed) {
	if( type == "x" ){
		var ds = sprite.scale.x - s;
		if( ds < 0 ){ sprite.scale.x += speed; }
		if( ds > 0 ){ sprite.scale.x -= speed; }
		if( Math.abs(ds) <= speed ){ sprite.scale.x = s; }
	}else{
		var ds = sprite.scale.y - s;
		if( ds < 0 ){ sprite.scale.y += speed; }
		if( ds > 0 ){ sprite.scale.y -= speed; }
		if( Math.abs(ds) <= speed ){ sprite.scale.y = s; }
	}
}
	
//==============================
// * 往mog强制塞入drill作用方法
//==============================
if(Imported.MOG_SceneSkill){
	Window_SkillListM.prototype.drill_skillSpriteInit = Window_SkillList.prototype.drill_skillSpriteInit;
	Window_SkillListM.prototype.drill_drawAllItems = Window_SkillList.prototype.drill_drawAllItems;
	Window_SkillListM.prototype.drill_drawItem = Window_SkillList.prototype.drill_drawItem;
	Window_SkillListM.prototype.drill_s_drawBlock = Window_SkillList.prototype.drill_s_drawBlock;
	Window_SkillListM.prototype.drill_s_drawBackground = Window_SkillList.prototype.drill_s_drawBackground;
	Window_SkillListM.prototype.drill_s_drawIcon = Window_SkillList.prototype.drill_s_drawIcon;
	Window_SkillListM.prototype.drill_skillSpriteUpdate = Window_SkillList.prototype.drill_skillSpriteUpdate;
	Window_SkillListM.prototype.drill_button_move_to = Window_SkillList.prototype.drill_button_move_to;
	Window_SkillListM.prototype.drill_scale_move_to = Window_SkillList.prototype.drill_scale_move_to;
}


//==============================
// * 窗口指针闪烁块（覆写分配）
//==============================
var _drill_bSkillWin_menu_cursorRect = Window_SkillList.prototype.isCursorVisible;
var _drill_bSkillWin_battle_cursorRect = Window_BattleSkill.prototype.isCursorVisible;

if( !DrillUp.bSkillWin_use_cursorRect ){
	if( DrillUp.bSkillWin_use_in_menu ){
		Window_SkillList.prototype.isCursorVisible = function() {
			return false;
		}
		Window_BattleSkill.prototype.isCursorVisible = function() {
			_drill_bSkillWin_battle_cursorRect.call(this);
		};
		if(Imported.MOG_SceneSkill){
			Window_SkillListM.prototype.isCursorVisible = function(x, y, width, height) {
				return false;
			};
		}
	}
	if( DrillUp.bSkillWin_use_in_battle ){
		Window_BattleSkill.prototype.isCursorVisible = function() {
			return false;
		}
	}
}

