//-----------------------------------------------------------------------------
//  Galv's New Game Plus
//-----------------------------------------------------------------------------
//  For: RPGMAKER MV
//  GALV_NewGamePlus.js
//-----------------------------------------------------------------------------
//  2016-06-26 - Version 1.0 - release
//-----------------------------------------------------------------------------
// Terms can be found at:
// galvs-scripts.com
//-----------------------------------------------------------------------------

var Imported = Imported || {};
Imported.Galv_NewGamePlus = true;

var Galv = Galv || {};            // Galv's main object
Galv.NEWGP = Galv.NEWGP || {};        // Galv's stuff

//-----------------------------------------------------------------------------
/*:
 * @plugindesc Start a new game while keeping things from a previous playthrough.
 * 
 * @author Galv - galvs-scripts.com
 *
 * @param 命令文本
 * @desc 当该插件可用时，将显示在标题的文本
 * @default New Game+
 *
 * @param 帮助文本
 * @desc 选择用于该插件的保存存档时，顶部帮助框中显示的文本
 * @default Select a save file to start a New Game+
 *
 * @param 可用图标
 * @desc 该插件可用时，出现在存档栏的图标
 * @default 245
 *
 * @param 可用图标 XY
 * @desc 图标的位置
 * x,y
 * @default 40,40
 *
 * @param NGP图标
 * @desc 如果是新周目，出现在存档列表的图标
 * @default 190
 *
 * @param NGP Icon XY
 * @desc Position of the New Game+ icon in the save file list
 * @default 0,40
 *
 * @param --------------
 * @desc
 * @default
 *
 * @param 最大金币
 * @desc 玩家可以继承的最大金币量，-1为保留所有
 * @default -1
 *
 * @param 角色列表
 * @desc 逗号分隔的变量将保留在新周目
 * 将变量ID用作单个或连字符以分隔范围。
 * @default
 *
 * @param --------------
 * @desc
 * @default
 *
 * @param 变量类型
 * @desc 0 =仅变量列表中的变量
 * 1 =除变量列表外的所有变量
 * @default 0
 *
 * @param 变量列表
 * @desc 逗号分隔变量，将保持在新游戏
 * 使用变量ID为单一或破折号分隔范围。
 * @default
 *
 * @param 开关类型
 * @desc 0 =仅开关列表的开关
 * 1 = 开关列表外的所有开关
 * @default 0
 *
 * @param 开关列表
 * @desc Comma separated switches that will keep in new game+
 * Use a switch ID for single or a dash to separate a range.
 * @default
 *
 * @param 道具类型
 * @desc 0 = Only items in the item list
 * 1 = All items excluding the item list
 * @default 0
 *
 * @param 道具列表
 * @desc Comma separated item ids that will keep in new game+
 * Use an item ID for single or a dash to separate a range.
 * @default
 *
 * @param 武器类型
 * @desc 0 = Only weapons in the weapon list
 * 1 = All weapons excluding the weapon list
 * @default 0
 *
 * @param 武器列表
 * @desc Comma separated weapon ids that will keep in new game+
 * Use a weapon ID for single or a dash to separate a range.
 * @default
 *
 * @param 护甲类型
 * @desc 0 = Only armors in the armor list
 * 1 = All armors excluding the armor list
 * @default 0
 *
 * @param 护甲列表
 * @desc Comma separated armor ids that will keep in new game+
 * Use an armor ID for single or a dash to separate a range.
 * @default
 *
 * @help
 *   Galv's New Game Plus
 * ----------------------------------------------------------------------------
 * 这个插件创造了多周目系统，
 * 允许玩家在重玩时使用之前的某些东西
 *
 * ----------------------------------------------------------------------------
 * HOW IS WORKS
 * ----------------------------------------------------------------------------
 * 在游戏过程中，当玩家到达您希望他们能够将其保存数据用
 * 于New Game +的程度（通常在玩家完成游戏时完成）时，
 * 请使用SCRIPT事件命令:
 *
 *     Galv.NEWGP.activate();
 *
 * 这将激活当前游戏中的新游戏+的能力。在这个脚本调用之后，玩家需要
 * 之后，玩家将需要保存他们的游戏。
 *
 * 下次玩家出现在标题屏幕上时，如果任何保存的游戏已经被上面的脚本调
 * 用激活，“New Game+”选项就会出现。
 * 这将把玩家带到保存文件列表中，玩家可以从已激活的保存游戏中进行选
 * 择，其中将有一个图标，如“图标”插件设置中指定的那样。
 * 
 *
 * 当一个新的game+启动时，插件设置中指定的数据将从所选择的保存文
 * 件中传输到游戏中。
 *
 * ----------------------------------------------------------------------------
 * PLUGIN SETTINGS
 * ----------------------------------------------------------------------------
 * Variables
 * ---------
 * 变量列表设置以及变量类型设置控制将哪些变量从保存文件传输到新游戏
 * 
 *
 * Example Variable List:
 * 34,56,90            // 一个包含3个变量的列表
 * 1-20,60-80          // 范围在1-20和60-80之间的变量列表
 * 1-20,60-80,56,90    // 2个单独变量加2组范围内的变量
 *
 * Example Variable Type:
 * 0  // 仅继承列表里的变量
 * 1  // 仅继承列表以外的变量
 *
 * Switches
 * --------
 * 在Rpg Maker中，当你启动一款新游戏时，所有的开关都默认为OFF。开
 * 关的工作方式与上面相同，使用开关列表和开关类型插件设置控制哪些开关
 * 被继承。
 *
 * Actors
 * ------
 * 角色设置的工作原理与上面一样，继承角色存档文件的角色数据。但是，
 * 角色没有“type”设置——列表包含您希望从存档中保留的所有角色
 * 的id。
 *
 * 但是请记住，当你在游戏中有一个“更换队伍角色”事件命令
 * 时，如果你在向队伍添加一个角色时勾选了“初始化”框，那么它将会
 * 将该角色恢复到初始等级。
 *
 * Items / Weapons / Armors
 * ------------------------
 * 这些设置也可以像上面的变量一样使用列表中项目的ID。
 * 
 *
 * ----------------------------------------------------------------------------
 *   SCRIPT CALLS
 * ----------------------------------------------------------------------------
 *
 *     Galv.NEWGP.activate();   // 激活该插件的功能
 *                              // (玩家需要在此之后保存)
 *
 * ----------------------------------------------------------------------------
 *   CONDITIONAL BRANCH - SCRIPT:
 * ----------------------------------------------------------------------------
 *
 *     Galv.NEWGP.isReplay()       // 如果玩家在玩新周目
 *     Galv.NEWGP.isReplay(x)      // 如果是第x周目
 *     Galv.NEWGP.isReplay(a,b)    // 如果是a到b之间的周目
 *
 *     Galv.NEWGP.isActive()       // 如果new game+在当前游戏处于活跃
 *
 * ----------------------------------------------------------------------------
 *   CONTROL VARIABLES - SCRIPT
 * ----------------------------------------------------------------------------
 *
 *    $gameSystem.newGamePlus   // 获得当前存档的周目数
 *                              // 
 */

//-----------------------------------------------------------------------------
//  CODE STUFFS
//-----------------------------------------------------------------------------

(function() {

Galv.NEWGP.newGameTxt = PluginManager.parameters('Galv_NewGamePlus')["Command Text"];
Galv.NEWGP.helpTxt = PluginManager.parameters('Galv_NewGamePlus')["Help Text"];
Galv.NEWGP.icon = Number(PluginManager.parameters('Galv_NewGamePlus')["Active Icon"]);
Galv.NEWGP.iconNGP = Number(PluginManager.parameters('Galv_NewGamePlus')["NGP Icon"]);
Galv.NEWGP.maxGold = Number(PluginManager.parameters('Galv_NewGamePlus')["Max Gold"]);

var txt = PluginManager.parameters('Galv_NewGamePlus')["Active Icon XY"].split(",");
Galv.NEWGP.iconXY = [Number(txt[0]),Number(txt[1])];

var txt = PluginManager.parameters('Galv_NewGamePlus')["NGP Icon XY"].split(",");
Galv.NEWGP.iconNGPXY = [Number(txt[0]),Number(txt[1])];


Galv.NEWGP.sortNumber = function(a,b) {
    return a - b;
};

Galv.NEWGP.makeList = function(string) {
	var list = string.split(",");
	var finalList = {'ids':[], 'ranges':[]};
	
	for (var i = 0; i < list.length; i++) {
		var number = Number(list[i]);
		if (isNaN(list[i])) {
			// create range
			var range = list[i].split("-");
			range = [Number(range[0]),Number(range[1])];
			range.sort(Galv.NEWGP.sortNumber);
			finalList.ranges.push(range);
		} else {
			// Add number
			finalList.ids.push(number);
		};
	}
	return finalList;
};


Galv.NEWGP.varType = Number(PluginManager.parameters('Galv_NewGamePlus')["Variable Type"]);
Galv.NEWGP.varList = Galv.NEWGP.makeList(PluginManager.parameters('Galv_NewGamePlus')["Variable List"]);

Galv.NEWGP.swiType = Number(PluginManager.parameters('Galv_NewGamePlus')["Switch Type"]);
Galv.NEWGP.swiList = Galv.NEWGP.makeList(PluginManager.parameters('Galv_NewGamePlus')["Switch List"]);

Galv.NEWGP.itemType = Number(PluginManager.parameters('Galv_NewGamePlus')["Item Type"]);
Galv.NEWGP.itemList = Galv.NEWGP.makeList(PluginManager.parameters('Galv_NewGamePlus')["Item List"]);

Galv.NEWGP.weaponType = Number(PluginManager.parameters('Galv_NewGamePlus')["Weapon Type"]);
Galv.NEWGP.weaponList = Galv.NEWGP.makeList(PluginManager.parameters('Galv_NewGamePlus')["Weapon List"]);

Galv.NEWGP.armorType = Number(PluginManager.parameters('Galv_NewGamePlus')["Armor Type"]);
Galv.NEWGP.armorList = Galv.NEWGP.makeList(PluginManager.parameters('Galv_NewGamePlus')["Armor List"]);

Galv.NEWGP.actorList = Galv.NEWGP.makeList(PluginManager.parameters('Galv_NewGamePlus')["Actor List"]);


Galv.NEWGP.activate = function() {
	$gameSystem.gameCompleted = true;
};

Galv.NEWGP.buildFinishedSaveList = function() {
	Galv.NEWGP.games = false;
	var count = DataManager.maxSavefiles();
	
	for (var i = 0; i < count; i++) {
		var valid = DataManager.isThisGameFile(i + 1);
		if (valid) {
			var game = DataManager.buildNewGamePlusData(i + 1);
			if (game.System.gameCompleted) {
				Galv.NEWGP.games = Galv.NEWGP.games || {};
				Galv.NEWGP.games[i + 1] = game;
			};
		};
	};
};

Galv.NEWGP.setupNewGamePlus = function() {
	var game = Galv.NEWGP.newGameSaveFile;
	
	// GAME SYSTEM
	$gameSystem = game.System;             // Duplicate Game System
	$gameSystem.newGamePlus += 1;          // Increase new game plus number
	$gameSystem.gameCompleted = false;     // Remove completed.
	Galv.NEWGP.newGameSaveFile = 0;
	
	// GAME PARTY
	var maxGold = Galv.NEWGP.maxGold < 0 ? game.Party._gold : Galv.NEWGP.maxGold;
	$gameParty._gold = Math.min(game.Party._gold,maxGold);
	
	// GAME VARIABLES
	this.setVariables(game);
	
	// GAME SWITCHES
	this.setSwitches(game);

	// GAME ACTORS
	this.setActors(game);
	
	// GAME ITEMS
	this.setItems(game,'item');
	this.setItems(game,'weapon');
	this.setItems(game,'armor');

};

// GAME VARIABLES
Galv.NEWGP.setVariables = function(game) {
	if (Galv.NEWGP.varType == 0) { // Only transfer List
		for (var i = 0; i < Galv.NEWGP.varList.ids.length; i++) {    // constants
			$gameVariables._data[Galv.NEWGP.varList.ids[i]] = game.Variables._data[Galv.NEWGP.varList.ids[i]];
		};
		for (var i = 0; i < Galv.NEWGP.varList.ranges.length; i++) { // ranges
			var range = Galv.NEWGP.varList.ranges[i];
			for (var r = range[0]; r <= range[1]; r++) {
				$gameVariables._data[r] = game.Variables._data[r];
			};
		};
	} else if (Galv.NEWGP.varType == 1) { // Transfer all EXCEPT list
		// change ALL variables
		$gameVariables = game.Variables;
		
		// then set excluded variables back to 0
		for (var i = 0; i < Galv.NEWGP.varList.ids.length; i++) {    // constants
			$gameVariables._data[Galv.NEWGP.varList.ids[i]] = 0;
		};
		for (var i = 0; i < Galv.NEWGP.varList.ranges.length; i++) { // ranges
			var range = Galv.NEWGP.varList.ranges[i];
			for (var r = range[0]; r <= range[1]; r++) {
				$gameVariables._data[r] = 0;
			};
		};
	};
};

// GAME SWITCHES
Galv.NEWGP.setSwitches = function(game) {
	if (Galv.NEWGP.swiType == 0) { // Only transfer List
		for (var i = 0; i < Galv.NEWGP.swiList.ids.length; i++) {    // constants
			// change all constant variables
			$gameSwitches._data[Galv.NEWGP.swiList.ids[i]] = game.Switches._data[Galv.NEWGP.swiList.ids[i]];
		};
		for (var i = 0; i < Galv.NEWGP.swiList.ranges.length; i++) { // ranages
			var range = Galv.NEWGP.swiList.ranges[i];
			for (var r = range[0]; r <= range[1]; r++) {
				$gameSwitches._data[r] = game.Switches._data[r];
			};
		};
	} else if (Galv.NEWGP.swiType == 1) { // Transfer all EXCEPT list
		// change ALL variables
		$gameSwitches = game.Switches;
		
		// then set excluded variables back to 0
		for (var i = 0; i < Galv.NEWGP.swiList.ids.length; i++) {    // constants
			$gameSwitches._data[Galv.NEWGP.swiList.ids[i]] = false;
		};
		for (var i = 0; i < Galv.NEWGP.swiList.ranges.length; i++) { // ranges
			var range = Galv.NEWGP.swiList.ranges[i];
			for (var r = range[0]; r <= range[1]; r++) {
				$gameSwitches._data[r] = false;
			};
		};
	};
};

// GAME ACTORS
Galv.NEWGP.setActors = function(game) {
	for (var i = 0; i < Galv.NEWGP.actorList.ids.length; i++) {    // constants
		$gameActors._data[Galv.NEWGP.actorList.ids[i]] = game.Actors._data[Galv.NEWGP.actorList.ids[i]];
	};
	for (var i = 0; i < Galv.NEWGP.actorList.ranges.length; i++) { // ranges
		var range = Galv.NEWGP.actorList.ranges[i];
		for (var r = range[0]; r <= range[1]; r++) {
			$gameActors._data[r] = game.Actors._data[r];
		};
	};
};


// GAME ITEMS / WEAPONS / ARMORS
Galv.NEWGP.setItems = function(game, iType) {
	var type = Galv.NEWGP[iType + 'Type'];
	var list = Galv.NEWGP[iType + 'List'];
	
	switch (iType) {
		case 'item':
			var itemBox = '_items';
			var itemData = $dataItems;
			break;
		case 'weapon':
			var itemBox = '_weapons';
			var itemData = $dataWeapons;
			break;
		case 'armor':
			var itemBox = '_armors';
			var itemData = $dataArmors;
			break;
	};
	
	if (type == 0) { // Only transfer List
		for (var i = 0; i < list.ids.length; i++) {    // constants
			var itemId = list.ids[i];
			var itemAmount = game.Party[itemBox][itemId];
			if (itemAmount) $gameParty.gainItem(itemData[itemId], itemAmount);
		};
		for (var i = 0; i < list.ranges.length; i++) { // ranges
			var range = list.ranges[i];
			for (var r = range[0]; r <= range[1]; r++) {
				var itemAmount = game.Party[itemBox][r];
				if (itemAmount) $gameParty.gainItem(itemData[r], itemAmount);
			};
		};
	} else if (type == 1) { // Transfer all EXCEPT list
		// change ALL items
		$gameParty[itemBox] = game.Party[itemBox];
		
		// then set excluded variables back to 0
		for (var i = 0; i < list.ids.length; i++) {    // constants
			var itemId = list.ids[i];
			if (itemAmount) $gameParty.gainItem(itemData[itemId], -game.Party[itemBox][itemId]);
		};
		for (var i = 0; i < list.ranges.length; i++) { // ranges
			var range = list.ranges[i];
			for (var r = range[0]; r <= range[1]; r++) {
				$gameParty.gainItem(itemData[r], -game.Party[itemBox][r]);
			};
		};
	};
};



Galv.NEWGP.isActive = function() {
	return $gameSystem.gameCompleted;
};

Galv.NEWGP.isReplay = function(a,b) {
	if (!a && !b) {
		return $gameSystem.newGamePlus > 0;
	} else if (!b) {
		return $gameSystem.newGamePlus == b;
	} else {
		if (b < 0) {
			// infinite
			return $gameSystem.newGamePlus > a;
		} else {
			var array = [a,b];
			array.sort(Galv.NEWGP.sortNumber);
			return ($gameSystem.newGamePlus > array[0] && $gameSystem.newGamePlus < array[1]);
		};
	};
};


// DATA MANAGER
//-----------------------------------------------------------------------------

Galv.NEWGP.DataManager_setupNewGame = DataManager.setupNewGame;
DataManager.setupNewGame = function() {
	Galv.NEWGP.DataManager_setupNewGame.call(this);
	if (Galv.NEWGP.newGameSaveFile) Galv.NEWGP.setupNewGamePlus();
};

DataManager.buildNewGamePlusData = function(savefileId) {
	var newGameData = {};
    var globalInfo = this.loadGlobalInfo();
    if (this.isThisGameFile(savefileId)) {
        var json = StorageManager.load(savefileId);
        newGameData = this.buildViewableSaveContents(JsonEx.parse(json));
        return newGameData;
    } else {
        return false;
    }
};

DataManager.buildViewableSaveContents = function(contents) {
	var obj = {};
    obj.System        = contents.system;
    obj.Switches      = contents.switches;
    obj.Variables     = contents.variables;
    obj.Actors        = contents.actors;
    obj.Party         = contents.party;
	return obj;
};

Galv.NEWGP.DataManager_makeSavefileInfo = DataManager.makeSavefileInfo;
DataManager.makeSavefileInfo = function() {
	var info = Galv.NEWGP.DataManager_makeSavefileInfo.call(this);
	info.newGamePlus = $gameSystem.newGamePlus;
	info.gameCompleted = $gameSystem.gameCompleted;
	return info;
};


// GAME SYSTEM
//-----------------------------------------------------------------------------

Galv.NEWGP.Game_System_initialize = Game_System.prototype.initialize;
Game_System.prototype.initialize = function() {
	Galv.NEWGP.Game_System_initialize.call(this);
	this.newGamePlus = 0;  // Number of times game has been new game plussed
	this.gameCompleted = false;  //
};


// SCENE TITLE
//-----------------------------------------------------------------------------

Galv.NEWGP.Scene_Title_create = Scene_Title.prototype.create;
Scene_Title.prototype.create = function() {
	Galv.NEWGP.buildFinishedSaveList();
	Galv.NEWGP.newGameSaveFile = 0;
	Galv.NEWGP.Scene_Title_create.call(this);
	this._commandWindow.setHandler('newGamePlus',  this.commandNewGamePlus.bind(this));
};

Scene_Title.prototype.commandNewGamePlus = function() {
	this._commandWindow.close();
    SceneManager.push(Scene_LoadNGPlus);
};


// WINDOW TITLE COMMAND
//-----------------------------------------------------------------------------

Galv.NEWGP.Window_TitleCommand_makeCommandList = Window_TitleCommand.prototype.makeCommandList;
Window_TitleCommand.prototype.makeCommandList = function() {
    if (Galv.NEWGP.games) this.addCommand(Galv.NEWGP.newGameTxt, 'newGamePlus'); // Show command if any save file is "completed"
    Galv.NEWGP.Window_TitleCommand_makeCommandList.call(this);
};


// WINDOW SAVE FILE LIST
//-----------------------------------------------------------------------------

Galv.NEWGP.Window_SavefileList_drawContents = Window_SavefileList.prototype.drawContents;
Window_SavefileList.prototype.drawContents = function(info, rect, valid) {
	Galv.NEWGP.Window_SavefileList_drawContents.call(this,info,rect,valid);
	this.drawNewGamePlus(info,rect.x,rect.y,rect.width);
};

Window_SavefileList.prototype.drawNewGamePlus = function(info, x, y, width) {
	if (info.gameCompleted) {
		// completed game icon
		this.drawIcon(Galv.NEWGP.icon,x + Galv.NEWGP.iconXY[0],y + Galv.NEWGP.iconXY[1]);
	}
	var NGPnumber = info.newGamePlus ? info.newGamePlus : 0;
	if (NGPnumber > 0) {
		// new game plus icon
		this.drawIcon(Galv.NEWGP.iconNGP,x + Galv.NEWGP.iconNGPXY[0],y + Galv.NEWGP.iconNGPXY[1]);
		// draw number
		if (NGPnumber > 1) {
			this.contents.fontSize = 16;
			this.drawText(NGPnumber,x + Galv.NEWGP.iconNGPXY[0] + 3,y + Galv.NEWGP.iconNGPXY[1] + 10,Window_Base._iconWidth,'right');
			this.resetFontSettings();
		};
	}
};

})();


// Scene_LoadNGPlus
//-----------------------------------------------------------------------------

function Scene_LoadNGPlus() {
    this.initialize.apply(this, arguments);
}

Scene_LoadNGPlus.prototype = Object.create(Scene_File.prototype);
Scene_LoadNGPlus.prototype.constructor = Scene_LoadNGPlus;

Scene_LoadNGPlus.prototype.helpWindowText = function() {
    return Galv.NEWGP.helpTxt;
};

Scene_LoadNGPlus.prototype.firstSavefileIndex = function() {
    return 0;
};

Scene_LoadNGPlus.prototype.onSavefileOk = function() {
	if (Galv.NEWGP.games[this._listWindow._index + 1]) {
		Galv.NEWGP.newGameSaveFile = Galv.NEWGP.games[this._listWindow._index + 1];
		SoundManager.playLoad();
		DataManager.setupNewGame();
		this.fadeOutAll();
		SceneManager.goto(Scene_Map);
	} else {
		SoundManager.playBuzzer();
		this.activateListWindow();
	};
};